import bson
import base64
from datetime import datetime

from django.db import models
from django.contrib.auth import get_user_model


User = get_user_model()


class Employee(models.Model):
    id = models.IntegerField(db_column="ID", primary_key=True)
    first_name = models.CharField(db_column="FIRSTNAME", max_length=64)
    last_name = models.CharField(db_column="LASTNAME", max_length=64, null=True)
    mid_name = models.CharField(db_column="MIDNAME", max_length=64, null=True)
    ssno = models.CharField(db_column="SSNO", max_length=13, null=True)
    lastchanged = models.DateTimeField(db_column="LASTCHANGED", max_length=13, null=True)
    visitor = models.IntegerField(db_column="VISITOR")
    allowed_visitors = models.IntegerField(db_column="ALLOWEDVISITORS")
    asset_group_id = models.IntegerField(db_column="ASSET_GROUPID", null=True)
    lnl_db_id = models.IntegerField(db_column="LNL_DBID")
    guard = models.IntegerField(db_column="GUARD")
    segment_id = models.IntegerField(db_column="SEGMENTID")

    class Meta:
        db_table = "EMP"


class EmployeeField(models.Model):
    FIELDS_LIST = [
        ( "id", "ID" ),
        ( "first_name", "First Name" ),
        ( "mid_name", "Middle Name" ),
        ( "last_name", "Last Name" ),
        ( "ssno", "SSNO" ),
        ( "lastchanged", "Last Changed" ),
        ( "visitor", "Visitor" ),
        ( "allowed_visitors", "Allowed Visitors" ),
        ( "asset_group_id", "Asset Group ID" ),
        ( "lnl_db_id", "LNL DB ID" ),
        ( "guard", "Guard" ),
        ( "segment_id", "Segment ID" ),
        ( "badges", "Badges" ),
        ( "badges__id", "Badge ID" ),
    ]

    field = models.CharField(max_length=128, choices=FIELDS_LIST, unique=True)
    searchable = models.BooleanField(default=True)
    filterable = models.BooleanField(default=True)

    def __str__(self):
        return dict(EmployeeField.FIELDS_LIST)[self.field]


class Badge(models.Model):
    id = models.IntegerField(db_column="ID", primary_key=True)
    # emp_id = models.IntegerField(db_column="EMPID")
    emp = models.ForeignKey(Employee, db_column="EMPID", related_name="badges", on_delete=models.CASCADE)
    type = models.IntegerField(db_column="TYPE")
    embossed = models.IntegerField(db_column="EMBOSSED")
    activate = models.DateTimeField(db_column="ACTIVATE")
    deactivate = models.DateTimeField(db_column="DEACTIVATE")
    status = models.IntegerField(db_column="STATUS")
    prints = models.IntegerField(db_column="PRINTS")
    issue_code = models.IntegerField(db_column="ISSUECODE")
    apbexempt = models.IntegerField(db_column="APBEXEMPT")
    last_print = models.DateTimeField(db_column="LASTPRINT", null=True)
    last_changed = models.DateTimeField(db_column="LASTCHANGED", null=True)
    use_limit = models.IntegerField(db_column="USELIMIT", null=True)
    extend_strike_held = models.IntegerField(db_column="EXTEND_STRIKE_HELD")
    passage_mode = models.IntegerField(db_column="PASSAGE_MODE")
    deadbolt_override = models.IntegerField(db_column="DEADBOLT_OVERRIDE", null=True)
    badge_key = models.IntegerField(db_column="BADGEKEY")
    pin = models.BinaryField(db_column="PIN", max_length=50, null=True)
    dest_exempt = models.IntegerField(db_column="DEST_EXEMPT")
    two_man_type = models.IntegerField(db_column="TWO_MAN_TYPE", null=True)
    default_floor = models.IntegerField(db_column="DEFAULT_FLOOR", null=True)
    descriptor_flag = models.IntegerField(db_column="DESCRIPTOR_FLAG")
    default_door = models.IntegerField(db_column="DEFAULT_DOOR")
    extended_id = models.CharField(db_column="EXTENDED_ID", max_length=64, null=True)
    direct_key_toolkit = models.IntegerField(db_column="DIRECT_KEY_TOOLKIT")

    class Meta:
        db_table = "BADGE"


class BadgeField(models.Model):
    FIELDS_LIST = [
        ( "id", "ID"),
        ( "type", "Type" ),
        ( "embossed", "Embossed" ),
        ( "activate", "Activate" ),
        ( "deactivate", "Deactivate" ),
        ( "status", "Status" ),
        ( "prints", "Prints" ),
        ( "issue_code", "Issue code" ),
        ( "apbexempt", "Apbexempt" ),
        ( "last_print", "Last print" ),
        ( "last_changed", "Last changed" ),
        ( "use_limit", "Use limit" ),
        ( "extend_strike_held", "Extend strike held" ),
        ( "passage_mode", "Passage mode" ),
        ( "deadbolt_override", "Deadbolt Override" ),
        ( "badge_key", "Badge key" ),
        ( "pin", "PIN" ),
        ( "dest_exempt", "Dest exempt" ),
        ( "two_man_type", "Two man type" ),
        ( "default_floor", "Defaulty floor" ),
        ( "descriptor_flag", "Descriptor flag" ),
        ( "default_door", "Default door" ),
        ( "extended_id", "Extended ID" ),
        ( "direct_key_toolkit", "Direct key toolkit" )
    ]

    field = models.CharField(max_length=128, choices=FIELDS_LIST, unique=True)
    searchable = models.BooleanField(default=True)
    filterable = models.BooleanField(default=True)

    def __str__(self):
        return dict(BadgeField.FIELDS_LIST)[self.field]


class BadgeStat(models.Model):
    id = models.IntegerField(db_column="ID", primary_key=True)
    name = models.CharField(db_column="NAME", max_length=32, null=True)

    class Meta:
        db_table = "BADGSTAT"
        ordering = [ "id" ]


class UpdateBadge(models.Model):
    badge_id = models.IntegerField()
    status = models.CharField(max_length=128, choices=[ (1, "Activated"), (4, "Gaia_Deactivated") ])

    class Meta:
        ordering = [ "id" ]


class History(models.Model):
    action = models.CharField(max_length=128)
    status = models.CharField(max_length=128, default="OK")
    target = models.CharField(max_length=128)
    time = models.DateTimeField(auto_now=False, auto_now_add=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    detail = models.TextField()

    class Meta:
        verbose_name_plural = "Histories"

    def __str__(self):
        return f"[{ self.time.isoformat() }] [{ self.status }] [{ self.user.username }] [{ self.target }] { self.action }"


class License(models.Model):
    key = models.TextField()
    active = models.BooleanField(default=True)

    def __str__(self):
        buf = base64.b64decode(self.key)
        obj = bson.loads(buf)
        now = datetime.now()
        exp = datetime.fromisoformat(obj["exp"])
        note = obj["note"] if "note" in obj else ""
        return f"[ { '  ACTIVE' if self.active else 'INACTIVE' } ] [{ 'EXPIRED' if exp<now else 'VALID' }] { note }"
