import json
import threading
from pathlib import Path

import wmi, pythoncom

from django.db.models import Q
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.permissions import IsAuthenticated, IsAuthenticatedOrReadOnly
from rest_framework.pagination import LimitOffsetPagination
from rest_framework.response import Response
from rest_framework import viewsets, status, filters, generics

from .serializers import EmployeeSerializer, BadgeSerializer, BadgeStatSerializer, UpdateBadgeSerializer
from .models import Employee, EmployeeField, Badge, BadgeStat, UpdateBadge, History


coInitialized = dict()

class EmployeeSetPagination(LimitOffsetPagination):
    max_limit = 50


class EmployeeViewSet(viewsets.ModelViewSet):
    queryset = Employee.objects.using("mssql").all()
    pagination_class = EmployeeSetPagination
    serializer_class = EmployeeSerializer
    permission_classes = ( IsAuthenticated, )
    http_method_names = ( "get", "head" )

    filter_backends = ( DjangoFilterBackend, filters.SearchFilter )
    filterset_fields = set()
    search_fields = set()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.filterset_fields = tuple([ r.field for r in EmployeeField.objects.filter(~Q(field__iexact="badges") & Q(filterable=True)).all() ])
        self.search_fields = tuple([ r.field for r in EmployeeField.objects.filter(~Q(field__iexact="badges") & Q(searchable=True)).all() ])

    def list(self, request):
        result = super().list(request)

        history = History()

        history.action = "LIST"
        history.detail = "{}"

        history.target = "Employee"
        history.user = request.user
        history.save()

        return result

    def retrieve(self, request, pk=None):
        result = super().retrieve(request, pk)

        history = History()

        history.action = "DETAIL"
        history.detail = json.dumps({ "pk": pk })

        history.target = "Employee"
        history.user = request.user
        history.save()

        return result


class BadgeViewSet(viewsets.ModelViewSet):
    queryset = Badge.objects.using("mssql").all()
    serializer_class = BadgeSerializer
    permission_classes = ( IsAuthenticated, )
    http_method_names = ( "get", "head" )

    def list(self, request):
        result = super().list(request)

        history = History()

        history.action = "LIST"
        history.detail = "{}"

        history.target = "Badge"
        history.user = request.user
        history.save()

        return result

    def retrieve(self, request, pk=None):
        result = super().retrieve(request, pk)

        history = History()

        history.action = "DETAIL"
        history.detail = json.dumps({ "pk": pk })

        history.target = "Badge"
        history.user = request.user
        history.save()

        return result


class BadgeStatViewSet(viewsets.ModelViewSet):
    queryset = BadgeStat.objects.using("mssql").all()
    serializer_class = BadgeStatSerializer
    permission_classes = ( IsAuthenticated, )
    http_method_names = ( "get", "head" )


class UpdateBadgeViewSet(viewsets.GenericViewSet, generics.CreateAPIView):
    queryset = UpdateBadge.objects.all()
    serializer_class = UpdateBadgeSerializer

    def create(self, request, *args, **kwargs):
        history = History()
        history.user = request.user
        history.target = "Badge"
        history.action = "UPDATE"

        serializer = UpdateBadgeSerializer(data=request.data)
        if not serializer.is_valid():
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

        badge_id = serializer.data["badge_id"]
        badge_status = serializer.data["status"]

        badge = Badge.objects.using("mssql").filter(id=badge_id).first()
        if not badge:
            history.status = "ERROR"
            history.detail = json.dumps({ "pk": badge_id, "to": badge_status, "message": "Badge not found" })
            history.save()

            return Response(status=status.HTTP_404_NOT_FOUND)

        badge_old_status = badge.status
        badge_key = badge.badge_key

        try:
            badge.status = badge_status
            badge.save()
        except Exception as e:
            history.status = "ERROR"
            history.detail = json.dumps({ "pk": badge_id, "to": badge_status, "message": e })
            history.save()

            return Response(status=status.HTTP_500_INTERNAL_SERVER_ERROR)

        history.detail = json.dumps({ "pk": badge_id, "from": badge_old_status, "to": badge_status, "badge_key": badge_key })
        history.save()

        if threading.get_ident() not in coInitialized or not coInitialized[threading.get_ident()]:
            pythoncom.CoInitialize()
            coInitialized[threading.get_ident()] = True

        BASE_DIR = Path(__file__).resolve(strict=True).parent.parent
        with open(f"{BASE_DIR}/wmi.json") as fp:
            wmi_conf = json.load(fp)

        args = []
        if "domain" in wmi_conf:
            args = [ wmi_conf["domain"] ]

        kwargs = dict()
        if "user" in wmi_conf:
            kwargs["user"] = wmi_conf["user"]

        if "password" in wmi_conf:
            kwargs["password"] = wmi_conf["password"]

        if "namespace" in wmi_conf:
            kwargs["namespace"] = wmi_conf["namespace"]

        c = wmi.WMI(*args, **kwargs)

        rs = [ str(r) for r in c.query(f"SELECT * FROM Lnl_Badge WHERE BADGEKEY={badge_key}") ]

        history = History()
        history.user = request.user
        history.target = "Badge"
        history.action = "WMI"
        history.detail = json.dumps(rs)
        history.save()

        return Response(status=status.HTTP_201_CREATED)
